#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "message.h"

/* all in milliseconds */

#define CW_DIT_TIME	1200/10		// ident at 10 WPM
#define CW_COUNT	1		// times to send

#define QRSS_DIT_TIME	6000		// dit interval
#define QRSS_COUNT	1		// times to send

#define KEY _BV(PB0)
#define FSK _BV(PB1)

enum element { DIT = 1, DAH = 3, CHAR_SPACE = 2, WORD_SPACE = 6, PAUSE = 10 };
enum speed { CW, QRSS };
enum modulation { MOD_OOK, MOD_FSK };

void init() {
	// keyed when low set high
	PORTB = 0;

	// output setup
	DDRB = KEY | FSK;
}

void key() { PORTB |= KEY; }
void unkey() { PORTB &= ~KEY; }

void shift() { PORTB |= FSK; }
void unshift() { PORTB &= ~FSK; }

void delay(int element, int speed) {
	int i;
	for(i = 0; i < element; i++) {
		switch(speed) {
			case CW:	_delay_ms(CW_DIT_TIME);		break;
			case QRSS:	_delay_ms(QRSS_DIT_TIME);	break;
		}
	}
}

void sendMessage(int modulation, int speed) {
	int i, j;

	if(modulation == MOD_FSK) key();
	delay(PAUSE, speed);

	for(i = 0; i < LENGTH; i++) {
		for(j = 0; j < 4; j++) {
			switch((pgm_read_byte(&morseData[i]) >> (6-(j*2))) & 0x03) {
				case 0:
					delay(WORD_SPACE, speed);
					break;
				case 1:
					delay(CHAR_SPACE, speed);
					break;
				case 2:
					(modulation == MOD_OOK)?( key() ):( shift() );
					delay(DIT, speed);
					(modulation == MOD_OOK)?( unkey() ):( unshift() );
					delay(DIT, speed);
					break;
				case 3:
					(modulation == MOD_OOK)?( key() ):( shift() );
					delay(DAH, speed);
					(modulation == MOD_OOK)?( unkey() ):( unshift() );
					delay(DIT, speed);
					break;
			}
		}
	}

	delay(PAUSE, speed);
	if(modulation == MOD_FSK) unkey();
}

int main() {
	int i;

	init();
	while(1) {
		// send in CW
		for(i = 0; i < CW_COUNT; i++) {
			sendMessage(MOD_OOK, CW);
		}
		
		// send at QRSS rate
		for(i = 0; i < QRSS_COUNT; i++) {
			sendMessage(MOD_FSK, QRSS);
		}
	}

	return 0; // unreached
}
